// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <array>
#include <vector>

#include <aoc/common/Interval.hpp>
#include <aoc/framework/Solver-types.hpp>

class DiskFragmenter
    : public Solver<int64_t, int64_t>
{
public:
    virtual const std::string getPuzzleName() const override;
    virtual const int getPuzzleDay() const override;
    virtual void processDataLine(const std::string& line) override;
    virtual void finish() override;
private:
    typedef std::vector<Interval> Intervals;
    typedef std::array<size_t, 10> DigitIndexArray;
    void moveFileBlocks(const std::string& line);
    void moveWholeFiles(const std::string& line);
    void AddNewEmptySpaces(Intervals& emptySpaces, const std::string& line, size_t& front, const size_t back,
        const int nBackBlocks, size_t& position);
    void UpdateEmptySpaceIndices(const Intervals& emptySpaces, DigitIndexArray& emptySpaceIndices);
    void moveBackFileForward(size_t& back, size_t& backIdNumber, const int nBackBlocks, Interval& emptySpace);
    void keepBackFile(const std::string& line, size_t& back, size_t& backIdNumber, const int nBackBlocks,
        size_t& position);
    unsigned int getDigit(const std::string& line, const size_t index) const;
    int64_t calcChecksumPart(const size_t idNumber, const int nBlocks, size_t& position) const;
};
