// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024-2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <bitset>
#include <vector>

#include <aoc/common/Grid.hpp>
#include <aoc/common/Point2.hpp>
#include <aoc/framework/LinesSolver.hpp>

class GuardGallivant
    : public LinesSolver<int64_t, int64_t>
{
public:
    virtual const std::string getPuzzleName() const override;
    virtual const int getPuzzleDay() const override;
    virtual void finish() override;
private:
    typedef Grid<bool> VisitGrid;
    typedef Grid<std::bitset<4>> PathGrid;
    static constexpr size_t getStartDirectionIndex();
    static constexpr char getStartChar();
    static constexpr char getObstructionChar();
    static constexpr char getEmptyChar();
    Point2 start_{};
    void tracePath();
    bool tracePathCycleTest(const Point2& startPosition, const size_t startDirectionIndex, PathGrid& mainPathGrid,
        PathGrid& cyclePathGrid);
    bool checkTurn(const Point2& current, const Point2& next, size_t& directionIndex, PathGrid& pathGrid);
    void visitPosition(const Point2& current, const size_t directionIndex, VisitGrid& visitGrid);
    void tracePosition(const Point2& current, const size_t directionIndex, PathGrid& pathGrid);
    void backtracePath(const Point2& current, const size_t directionIndex, PathGrid& pathGrid);
    void backtraceTurn(const Point2& current, const size_t reverseTurnDirectionIndex, PathGrid& pathGrid);
    size_t turnDirectionRight(const size_t currentDirectionIndex) const;
    size_t turnDirectionLeft(const size_t currentDirectionIndex) const;
    size_t revertDirection(const size_t currentDirectionIndex) const;
    bool isObstruction(const Point2& position) const;
};
