// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <array>
#include <vector>

#include <aoc/common/Point2.hpp>
#include <aoc/framework/Solver-types.hpp>

class RestroomRedoubt
    : public Solver<int64_t, int64_t>
{
public:
    RestroomRedoubt(const int width = 101, const int height = 103, const bool runPart2 = true);
    virtual const std::string getPuzzleName() const override;
    virtual const int getPuzzleDay() const override;
    virtual void processDataLine(const std::string& line) override;
    virtual void finish() override;
private:
    const int width_;
    const int height_;
    const int halfWidth_;
    const int halfHeight_;
    const bool runPart2_;
    std::array<int, 4> quadrants_;
    std::vector<std::pair<Point2, Point2>> robots_;
    static constexpr int getNPredictionSeconds();
    static constexpr int getEasterEggThreshold();
    void findEasterEgg();
};
