// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024-2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <array>
#include <iostream>

class Point2
{
public:
    static const Point2 left, right, up, down;
    static const Point2 upLeft, upRight, downLeft, downRight;
    /// <summary>
    /// The eight cardinal and diagonal directions starting down, rotating in
    /// positive direction.
    /// </summary>
    static const std::array<Point2, 8> directions;
    /// <summary>
    /// The four cardinal directions starting down, rotating in positive direction.
    /// </summary>
    static const std::array<Point2, 4> cardinalDirections;
    static Point2 getCardinalDirection(const char directionChar);
    int x, y;
    Point2();
    Point2(const int x, const int y);
    bool operator==(const Point2& rhs) const;
    bool operator!=(const Point2& rhs) const;
    bool operator<(const Point2& rhs) const;
    Point2 operator+(const Point2& rhs) const;
    Point2 operator-(const Point2& rhs) const;
    Point2 operator*(const int rhs) const;
    Point2 operator-() const;
    Point2& operator+=(const Point2& rhs);
    Point2& operator-=(const Point2& rhs);
    Point2& operator*=(const int rhs);
    int& operator[](size_t coordinateIndex);
};

std::ostream& operator<<(std::ostream& os, const Point2& rhs);
