// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024-2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <filesystem>
#include <fstream>
#include <iostream>
#include <string>
#include <memory>
#include <vector>

#include <aoc/framework/Solver.hpp>

class SolverEngine
{
public:
    SolverEngine(const std::vector<std::string>& inputPaths);

    template <typename T1, typename T2>
    void run(Solver<T1, T2>& solver)
    {
        std::cout << "\n--- Day " << solver.getPuzzleDay() << ": " << solver.getPuzzleName() << " ---\n";

        auto fullFilePath = tryGetValidFullInputFilePath(solver.getInputFileName());
        if (fullFilePath != "")
        {
            std::string line;
            std::ifstream inputFile{ fullFilePath };
            while (std::getline(inputFile, line))
            {
                solver.processDataLine(line);
            }
            inputFile.close();

            solver.finish();

            std::cout << "Part 1: " << solver.getResultPart1() << "\nPart 2: " << solver.getResultPart2() << std::endl;
        }
    }
private:
    std::vector<std::string> inputPaths_;
    std::filesystem::path tryGetValidFullInputFilePath(const std::string& inputFileName);
};
