// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024-2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aocTests/TestContext.hpp>

void TestContext::run(std::unique_ptr<Solver<int64_t, int64_t>>&& solver, const int64_t expected1,
    const int64_t expected2, const std::vector<std::string>& inputPaths)
{
    runGeneric(std::move(solver), expected1, expected2, inputPaths);
}

void TestContext::run(std::unique_ptr<Solver<int64_t, std::string>>&& solver, const int64_t expected1,
    const std::string& expected2, const std::vector<std::string>& inputPaths)
{
    runGeneric(std::move(solver), expected1, expected2, inputPaths);
}

void TestContext::run(std::unique_ptr<Solver<std::string, int64_t>>&& solver, const std::string& expected1,
    const int64_t expected2, const std::vector<std::string>& inputPaths)
{
    runGeneric(std::move(solver), expected1, expected2, inputPaths);
}

void TestContext::runPart1(std::unique_ptr<Solver<int64_t, int64_t>>&& solver, const int64_t expected,
    const std::vector<std::string>& inputPaths)
{
    runPart1Generic(std::move(solver), expected, inputPaths);
}

void TestContext::runPart1(std::unique_ptr<Solver<std::string, int64_t>>&& solver, const std::string& expected,
    const std::vector<std::string>& inputPaths)
{
    runPart1Generic(std::move(solver), expected, inputPaths);
}

void TestContext::runPart2(std::unique_ptr<Solver<int64_t, int64_t>>&& solver, const int64_t expected,
    const std::vector<std::string>& inputPaths)
{
    runPart2Generic(std::move(solver), expected, inputPaths);
}

std::vector<std::string> TestContext::getInputPaths() const
{
    return std::vector<std::string>{ "data", "../../../data", "../../../../data" };
}

std::vector<std::string> TestContext::getExampleInputPaths() const
{
    return std::vector<std::string>{ "data/examples", "../../../data/examples", "../../../../data/examples" };
}
